******************************************************
** PROJECT     : WIYN OTA DEVICE SEQUENCER CODE
** TITLE       : OTA DEVICE IN LAB TEST DEWAR
**             : 
** VERSION     : 003
** DATE        : 1/18/2006
** AUTHOR      : DAVE SAWYER
** HISTORY     : VERSION 1 -- DGS - Initial sequencer for STA1571 based on STA0510 ver.2 
**	       : Version 2 -- DGS - Changed SW for clocking independent of S3
**				  - Also set INTEG_TIME as external variable (sta1571.mod file)
**             : VERSION 3 -- DGS - Expanded for full OTA readout
**				    Added microsequencer for CDS signals 
** DESCRIPTION : First test of OTA device.
**             : COMPILE WITH ASM4 OR LATER.
** SIGNAL
** ASSIGNMENTS :
**             : P1      => CLK_OUT_00
**             : P2      => CLK_OUT_01
**             : P3      => CLK_OUT_02
**             : P4      => CLK_OUT_03
**             : RG      => CLK_OUT_04
**             : S1      => CLK_OUT_05
**             : S2      => CLK_OUT_06
**             : S3      => CLK_OUT_07
**             : SW      => CLK_OUT_27
**	       : SHUT    => CLK_OUT_31
**
***************************
** CLOCK BIT DEFINITIONS **
***************************
DEF #P1            B00000000000000000000000000000001
DEF #P2            B00000000000000000000000000000010
DEF #P3            B00000000000000000000000000000100
DEF #P4            B00000000000000000000000000001000
DEF #RG            B00000000000000000000000000010000  
DEF #S1            B00000000000000000000000000100000
DEF #S2            B00000000000000000000000001000000
DEF #S3            B00000000000000000000000010000000
DEF #SW            B00001000000000000000000000000000 
DEF #TST           B10000000000000000000000000000000
DEF #SHC           B10000000000000000000000000000000

******************************************************************************************************
** STA OTA Configuration *
*
*
*		             P->   4 22   4 22   4 22
*			     P->   433    433    433
*			     P->    111    111    111
*			         -----------------------
*       OTG|SW-2-1|S3-2-1|S3-2-1|S3-2-1|S3-2-1|S3-2-1....		
*        |   <-3 prescan pixels->    parallels dump into S2 and S1
*        |    OD
*       _|_   _|
*     _|   |_|
* RG-|_      |_
*      |       |
*     RD      OS
*
*  NOTE: PSL (standby low) is connected to P3 and P4
*        PSH (standby high) is connected to P1 and P2
*
* During integration P1 and P2 are high, so S1 must be low to prevent leakage from the parallels into
* the serials.  As part of the transition from the integration state to a serial transfer, S1 must be 
* set high.  Then S1 will be set low as the first serial charge transfer. Integrate on reset will occur
* when S2 is high, before S3 (and SW) are set high.
*
******************************************************************************************************

** PARALLEL CLOCK BIT PATTERNS for P1-P2-P3 clocking
** Parallel phases 1 and 2 high during integration
DEF #T_STATE_0     #P1 OR #P2             * Integrate state
DEF #T_STATE_1     #P2
DEF #T_STATE_2     #P2 OR #P3
DEF #T_STATE_3     #P3
DEF #T_STATE_4     #P1 OR #P3
DEF #T_STATE_5     #P1

** SERIAL CLOCK BIT PATTERNS for S1-S2-S2 clocking (charged transferred to S1 and S2)
DEF #S_STATE_0     #S2                    * This is the static state (integ or OT shift)
DEF #S_STATE_1     #S2 OR #S3 OR #SW
DEF #S_STATE_2     #S3 OR #SW                 
DEF #S_STATE_3     #S3 OR #SW OR #S1         
DEF #S_STATE_4     #S1           
DEF #S_STATE_5     #S1 OR #S2             * State for parallel transfers

** INTEGRATION AND INITIALIZATION CLOCKING VECTORS
DEF #INT_0      #T_STATE_0 OR #S_STATE_0 OR #RG
DEF #SHUT_OPEN  #INT_0 OR #SHC
DEF #SHUT_CLOSE #INT_0

** PARALLEL CLOCKING VECTORS
DEF #PAR_0      #T_STATE_0 OR #S_STATE_5  * First bring S1 high
DEF #PAR_1      #T_STATE_1 OR #S_STATE_5
DEF #PAR_2      #T_STATE_2 OR #S_STATE_5
DEF #PAR_3      #T_STATE_3 OR #S_STATE_5
DEF #PAR_4      #T_STATE_4 OR #S_STATE_5
DEF #PAR_5      #T_STATE_5 OR #S_STATE_5
DEF #PAR_6      #T_STATE_5 OR #S_STATE_0  *Put back to static serial state
* DEF #PAR_6      #T_STATE_0 OR #S_STATE_5  * Must bring P1 low during serial xfer
DEF #PAR_7      #T_STATE_1 OR #S_STATE_5  * Ready for serial xfer

** SERIAL CLOCKING VECTORS
DEF #SER_0      #S_STATE_0 OR #T_STATE_0
DEF #SER_1      #S_STATE_0 OR #T_STATE_0 OR #RG
DEF #SER_2      #S_STATE_1 OR #T_STATE_0
DEF #SER_3      #S_STATE_2 OR #T_STATE_0
DEF #SER_4      #S_STATE_3 OR #T_STATE_0
DEF #SER_5      #S_STATE_4 OR #T_STATE_0  * Video dump occurs here
DEF #SER_6      #S_STATE_5 OR #T_STATE_0

** SERIAL CLEARING VECTORS
DEF #CLR_0      #S_STATE_0 OR #T_STATE_0 OR #RG
DEF #CLR_1      #S_STATE_1 OR #T_STATE_0 OR #RG
DEF #CLR_2      #S_STATE_2 OR #T_STATE_0 OR #RG
DEF #CLR_3      #S_STATE_3 OR #T_STATE_0 OR #RG
DEF #CLR_4      #S_STATE_4 OR #T_STATE_0 OR #RG
DEF #CLR_5      #S_STATE_5 OR #T_STATE_0 OR #RG

**********************
** TIMING CONSTANTS **
**********************
DEF #THETA_OVERLAP  10              * 10 us read parallel phase overlaps (us)
DEF #RESET_OVERLAP  10              * 10 us reset parallel phase overlap (us)
DEF #RST_TIME       14              * Delay for Reset on time    (clk) 700 ns
DEF #RST_DLY        10              * Delay for signal to settle (clk) 500 ns
DEF #INT_TIME       80              * Delay for integration time (clk) about 4 us
DEF #SER_OVERLAP    14              * Delay serial clock overlap (clk) about 700 ns
DEF #CLR_OVERLAP     2              * 2 sequencer clocks = 100ns (clk)
DEF #CDSCMD0	    x0010B00	    * 500 Kpix/sec acquisition with DC clamp and reset on
DEF #CDSCMD1	    x0010B10	    * 500 Kpix/sec acquisition with DC clamp on
DEF #CDSCMD2	    x0010B20	    * 500 Kpix/sec acquisition
DEF #CDSCMD3	    x0010B30	    * 800 Kpix/sec acquisition
DEF #MODE_DELAY    100            * Delay to allow mode to be established on detector

**************************
** MODE BIT DEFINITIONS **
**************************
DEF #MOD_DISABLE    1       	* Set Cell to floating (disabled)
DEF #MOD_GS_SEL     2       	* Set the guide regions to active with video on (guide region readout)
DEF #MOD_ROW_SEL    3       	* Set all cells in a row to active with video on (full array readout)
DEF #MOD_ROW_RST    4       	* Reset the row loop counter to the first row
DEF #MOD_SHIFT      5       	* Set the science cells to active video off (OT shift)
DEF #MOD_STANDBY    6       	* Set all cells to standby (integration)
DEF #MOD_ACTIVE     7       	* Set all cells to active with video off

*******************************
** LOOP REGISTER DEFINITIONS **
*******************************
DEF #ROWS         0                 * NUMBER OF ROWS TO READOUT ((DET ROWS - SKIPROW) / ROWBIN)
DEF #COLS         1                 * NUMBER OF COLUMNS TO READOUT ((DET COLS - SKIPCOL) / COLBIN)
DEF #ROWBIN       2                 * BINNING FACTOR FOR ROWS
DEF #COLBIN       3                 * BINNING FACTOR FOR COLS
DEF #ROWSKIP      4                 * NUMBER OF ROWS TO SKIP FROM ORIGIN
DEF #COLSKIP      5                 * NUMBER OF COLS TO SKIP FROM ORIGIN
DEF #OTA_ROWS     6                 * Number of cell rows on OTA (typically 8)
DEF #WIPES        7                 * NUMBER OF WIPES PRIOR TO INTEGRATION 
** 
**************************************************************
** JUMP FLAG DEFINITIONS - BIT POSITION IN COMMAND REGISTER **
**************************************************************
DEF #SYNC        1                  * Jump on sync bit high
DEF #ITC         2                  * Jump on Integration time expired
DEF #START       3                  * Jump on start exposure flag set
DEF #CCDREADOUT  4                  * STRVCTR =  1: DO COMPLETE CCD CLEAR - INTEGRATE - READ
DEF #CCDCLEAR    5                  * STRVCTR =  2: DO CCD CLEAR ONLY
DEF #CCDREAD     6                  * STRVCTR =  4: DO CCD READ ONLY
DEF #VCTR_4      7                  * STRVCTR =  8: 
DEF #VCTR_5      8                  * STRVCTR = 16:
DEF #VCTR_6      9                  * STRVCTR = 32:
DEF #VCTR_7     10                  * STRVCTR = 64:
DEF #VCTR_8     11                  * STRVCTR =128:
DEF #CONT_RUN   12                  * USER BIT 1: LOOP INDEFINATELY ON TESTS
DEF #USR_2      13                  * Jump on user mode bit value 2
DEF #USR_3      14                  * Jump on user mode bit value 4
DEF #USR_4      15                  * Jump on user mode bit value 8
**
****************************
** EFR REGISTER CONSTANTS **
****************************
DEF #KILL_EXPFLG    x00000001       * Cancel start exposure flag using the EFR register
DEF #STRT_INTCNTR   x00000002       * Start the integration time counter using the EFR register
DEF #STOP_INTCNTR   x00000004       * Stop the integration time counter using the EFR register
DEF #LSR_AUTO_KILL  x00000100       * Enables automatic board select cancel after an LSR instruction
DEF #LSR_MAN_RESET  x00000200       * Disables automatic board select cancel. LSR will remain active until an LSR #DSLCT is issued
DEF #SET_SYNC_LOW   x00004000       * If enabled as master DHE, set sync bit low after sync delay
DEF #SET_SYNC_HIGH  x00008000       * If enabled as master DHE, set sync signal high immediately
DEF #KILLANDSYNC    x00004001       * Set sync low and kill start exposure flag
**
*****************************
** MODE REGISTER CONSTANTS **
*****************************
DEF #RESET       0                  * Mode register constant for board reset operation
DEF #READ        1                  * Mode register constant for read operation
DEF #WRITE16     2                  * Mode register constant for 16 bit write operation
DEF #WRITE32     3                  * Mode register constant for 32 bit write operation
**
**********************************
** LSR BOARD SELECT ASSIGNMENTS **
**********************************
DEF #DSLCT       0
DEF #MCB         1
DEF #CCDCLK      2
DEF #CCDACQ      8
**
** CCDCLK BOARD ADDRESS POINTERS
DEF #CLKPORT     x0000
DEF #FBIAS0123   x0001
DEF #FBIAS4567   x0002
DEF #OTAMODE     x0003
**
** CCDACQ BOARD ADDRESS POINTERS
DEF #CDSREG      x0000
DEF #DOPREG      x0001
DEF #BRSTCMD     x0004
DEF #SEQCMD      x0005
**
********************************
** PATTERN MEMORY DEFINITIONS **
********************************
** PATTERN MEMORY SEGMENTATION MAP
DEF #PATSEG_00      x0000             * Pattern memory origin - Initialization stuff
DEF #PATSEG_CLR     x0020             * Pattern memory origin - CCD CLEAR VECTORS
DEF #PATSEG_INIT    x0040             * Pattern memory origin - CCD READ INITIALIZATION
DEF #PATSEG_PAR     x0060             * Pattern memory origin - CCD PARALLEL SHIFT VECTORS
DEF #PATSEG_SER     x0080             * Pattern memory origin - CCD SERIAL SHIFT VECTORS
DEF #PATSEG_MOD     x00A0             * Pattern memory origin - OTA MODE VECTORS

ORG PAT #PATSEG_00                    * ********* GENERAL CONSTANTS KEPT IN PAT MEM ********* 
PAT #KILL_EXPFLG    #KILL_EXPFLG      * We can use the same label name here because they are in different contexts
PAT #STRT_INTCNTR   #STRT_INTCNTR     * The definition earlier points to a data value. The label here points to
PAT #STOP_INTCNTR   #STOP_INTCNTR     * a pattern memory address
PAT #CDSCMD0        #CDSCMD0          * Pattern defines the CDS microsequencer
PAT #CDSCMD1        #CDSCMD1          * Pattern defines the CDS microsequencer
PAT #CDSCMD2        #CDSCMD2          * Pattern defines the CDS microsequencer
PAT #CDSCMD3        #CDSCMD3          * Pattern defines the CDS microsequencer

*ORG PAT #PATSEG_CLR                  * ********* CCD CLEAR VECTORS ********* 
PAT #CLR_0          #CLR_0
PAT #CLR_1          #CLR_1
PAT #CLR_2          #CLR_2
PAT #CLR_3          #CLR_3
PAT #CLR_4          #CLR_4
PAT #CLR_5          #CLR_5

ORG PAT #PATSEG_INIT                  * ********* CCD INTEGRATION AND READ INITIALIZATION ********* 
PAT #INT_0          #INT_0
PAT #SHUT_OPEN      #SHUT_OPEN
PAT #SHUT_CLOSE     #SHUT_CLOSE

ORG PAT #PATSEG_PAR                   * ********* CCD PARALLEL SHIFT VECTORS ********* 
PAT #PAR_0          #PAR_0
PAT #PAR_1          #PAR_1
PAT #PAR_2          #PAR_2
PAT #PAR_3          #PAR_3
PAT #PAR_4          #PAR_4
PAT #PAR_5          #PAR_5
PAT #PAR_6          #PAR_6
PAT #PAR_7          #PAR_7

ORG PAT #PATSEG_SER                   * ********* CCD SERIAL SHIFT VECTORS ********* 
PAT #SER_0          #SER_0
PAT #SER_1          #SER_1
PAT #SER_2          #SER_2
PAT #SER_3          #SER_3
PAT #SER_4          #SER_4
PAT #SER_5          #SER_5
PAT #SER_6          #SER_6

ORG PAT #PATSEG_MOD                   * ************ OTA MODE VECTORS ****************
PAT #MOD_DISABLE    #MOD_DISABLE
PAT #MOD_GS_SEL     #MOD_GS_SEL
PAT #MOD_ROW_SEL    #MOD_ROW_SEL
PAT #MOD_ROW_RST    #MOD_ROW_RST
PAT #MOD_SHIFT      #MOD_SHIFT
PAT #MOD_STANDBY    #MOD_STANDBY
PAT #MOD_ACTIVE     #MOD_ACTIVE

*************************************
* PROGRAM CODE SEGMENT STARTS HERE **
*************************************
** EXECUTIVE ROUTINE
#INIT    DMS 200                     * Allow the PAN to finish abusing the bus.
#EXEC    JCB #TEST #CONT_RUN       * KLUDGE FOR THE START EXPOSURE PROBLEM
         JMP #EXEC

#TEST    CAL #CLRTEST
         JMP #EXEC

#CLRTEST LRB #ROWS                   * LOOP FOR N ROWS
         CAL #PAR_SHIFT              * DO ROW XFERS FOR N BINS
         LRB #COLS                   * LOOP FOR N COLS
         CAL #MARKER
         CAL #SER_READ               * DO COLUMN READ FOR N BINS
         LPE                         * END OF COLUMN LOOP
         LPE                         * END OF ROW LOOP
         RET

#MARKER  CAL #OPEN_S
         DSC 4
         CAL #CLOSE_S
         DSC 4
         RET

** READOUT ROUTINE

#JRDOUT LMR #WRITE32
        LRB #WIPES
        CAL #CLEAR                  * Wipe the chip the number of times defined by WIPES
        LPE
        CAL #INTEG                  * PUT THE DETECTOR BACK INTO INTEGRATE MODE
        CAL #OPEN_S
        LPP #STRT_INTCNTR
        LSR #MCB                    * START INTEGRATION TIMER RUNNING
        NOP                         * A bug in the MPU .. needs time to reset the flag
        NOP

#RD1    JCB #RD2 #ITC
        JMP #RD1

#RD2    CAL #CLOSE_S
        LMR #WRITE32                * HERE WHEN ITC HAS REACHED REQUIRED TIME
        LPP #STOP_INTCNTR
        LSR #MCB                    * STOP INTEGRATION TIMER
        CAL #ARRAY_READ
#        JCB #JRDOUT #CONT_RUN       * DO IT ALL AGAIN IF TOLD TO DO SO
        JMP #EXEC                   * CALL IT A DAY

#JCLEAR CAL #CLEAR
#        JCB #JCLEAR #CONT_RUN       * DO IT ALL AGAIN IF TOLD TO DO SO
        JMP #EXEC

#JREAD  CAL #ARRAY_READ
#        JCB #JREAD #CONT_RUN        * DO IT ALL AGAIN IF TOLD TO DO SO
        JMP #EXEC         
  
#OPEN_S LMR #WRITE32
        LDA #CLKPORT
        LPP #SHUT_OPEN
        LSR #CCDCLK
        RET
        
#CLOSE_S LMR #WRITE32
        LDA #CLKPORT
        LPP #SHUT_CLOSE
        LSR #CCDCLK
        RET

#CLEAR  CAL #RINIT                  * INITIALIZE FOR SERIAL READ
        LDA #OTAMODE
        LPP #MOD_ACTIVE             * Set all cells to active video off
        LSR #CCDCLK
        DUS #MODE_DELAY             * Allow time for mode to be established
        CAL #SER_CLEAR              * SWEEP THE SERIAL REGISTER
        LRB #ROWS                   * LOOP FOR N ROWS
        CAL #PAR_SHIFT              * DO ROW XFERS FOR N BINS
        DUS 1                       * Allow parallels to settle
        CAL #SER_CLEAR              * DO COLUMN CLEAR 
        LPE                         * END OF ROW LOOP
        RET
        
#INTEG  LMR #WRITE32                * SET 32 BIT WRITE MODE
        LDA #OTAMODE
        LPP #MOD_STANDBY            * Set the cells to standby mode
        LSR #CCDCLK
        LDA #CLKPORT
        LPP #INT_0
        LSR #CCDCLK                 * Establish the clock phasing for integration
        RET

#GS_READ   CAL #RINIT               * INITIALIZE FOR READ
	LDA #OTAMODE
	LPP #MOD_GS_SEL	    	    * Set all guide cells to active video on
	LSR #CCDCLK
        CAL #SER_CLEAR              * SWEEP THE SERIAL REGISTER
        LRB #ROWS                   * LOOP FOR N ROWS
        CAL #PAR_SHIFT              * DO ROW XFERS FOR N BINS
        LRB #COLS                   * LOOP FOR N COLS
        CAL #SER_READ               * DO COLUMN READ FOR N BINS
        LPE                         * END OF COLUMN LOOP
        LPE                         * END OF ROW LOOP
        CAL #INTEG                  * PUT THE DETECTOR BACK INTO INTEGRATE MODE
        RET
        
#ARRAY_READ   CAL #RINIT                  * INITIALIZE FOR READ
	LDA #OTAMODE
	LPP #MOD_ROW_RST	    * Reset the row pointer to the first row of cells
	LSR #CCDCLK
	LPP #MOD_ROW_SEL	    * Set all cells in row 0 to active video on
	LSR #CCDCLK
	LRB #OTA_ROWS		    * For all eight rows of cells
        CAL #SER_CLEAR              * SWEEP THE SERIAL REGISTER
        LRB #ROWS                   * LOOP FOR N ROWS
        CAL #PAR_SHIFT              * DO ROW XFERS FOR N BINS
        LRB #COLS                   * LOOP FOR N COLS
        CAL #SER_READ               * DO COLUMN READ FOR N BINS
        LPE                         * END OF COLUMN LOOP
        LPE                         * END OF ROW LOOP
        LPE                         * END OF OTA ROW LOOP
        CAL #INTEG                  * PUT THE DETECTOR BACK INTO INTEGRATE MODE
        RET
        
#SER_CLEAR  LMR #WRITE32
        LDA #CLKPORT
        LRB #COLS                   * Do all the columns 
        LPP #CLR_0 
        LSR #CCDCLK
        DSC #CLR_OVERLAP
        LPP #CLR_1
        LSR #CCDCLK
        DSC #CLR_OVERLAP
        LPP #CLR_2
        LSR #CCDCLK
        DSC #CLR_OVERLAP
        LPP #CLR_3
        LSR #CCDCLK
        DSC #CLR_OVERLAP
        LPP #CLR_4
        LSR #CCDCLK
        DSC #CLR_OVERLAP
        LPP #CLR_5
        LSR #CCDCLK
        DSC #CLR_OVERLAP
        LPE
        RET

#SER_READ  LMR #WRITE32
        LPP #CDSCMD2
        LDA #SEQCMD
        LSR #CCDACQ       *Fires the CDS microsequencer
        LDA #CLKPORT
*        LPP #SER_0        *S1=LO S2=HI S3=LO SW=LO  RG=HI 
*        LSR #CCDCLK                    
*        DSC #SER_OVERLAP   * Delay for reset pulse
        LPP #SER_1        *S1=LO S2=HI S3=LO SW=LO RG=HI 
        LSR #CCDCLK             
        DSC #RST_TIME     * Delay for Intergrate on reset 	
        LPP #SER_2        *S1=LO S2=HI S3=HI SW=HI RG=LO 
        LSR #CCDCLK             
        DSC #SER_OVERLAP
        LPP #SER_3        *S1=LO S2=LO S3=HI SW=HI RG=LO 
        LSR #CCDCLK             
        DSC #SER_OVERLAP
        LPP #SER_4        *S1=HI S2=LO S3=HI SW=HI RG=LO 
        LSR #CCDCLK             
        DSC #SER_OVERLAP
        LPP #SER_5        * VIDEO DUMP OCCURS HERE S1=HI S2=LO S3=LO SW=LO RG=LO
        LSR #CCDCLK             
        DSC #SER_OVERLAP     *Delay for integrate on signal	
        LPP #SER_6        *S1=HI S2=HI S3=LO SW=LO RG=LO 
        LSR #CCDCLK             
        DSC #SER_OVERLAP
        LPP #SER_0        *S1=LO S2=HI S3=LO SW=LO  RG=HI 
        LSR #CCDCLK                    
        DSC #SER_OVERLAP   * Delay for reset pulse
RET

#RINIT  RET

#PAR_SHIFT  LMR #WRITE32
        LDA #CLKPORT
        LRB #ROWBIN                 * DO THIS FOR ROW BINNING FACTOR
        LPP #PAR_0
        LSR #CCDCLK                 * P1=HI P2=HI P3=LO
        DUS #THETA_OVERLAP        
        LPP #PAR_1
        LSR #CCDCLK                 * P1=LO P2=HI P3=LO
        DUS #THETA_OVERLAP
        LPP #PAR_2
        LSR #CCDCLK                 * P1=LO P2=HI P3=HI
        DUS #THETA_OVERLAP
        LPP #PAR_3
        LSR #CCDCLK                 * P1=LO P2=LO P3=HI
        DUS #THETA_OVERLAP
        LPP #PAR_4
        LSR #CCDCLK                 * P1=HI P2=LO P3=HI
        DUS #THETA_OVERLAP
        LPP #PAR_5
        LSR #CCDCLK                 * P1=HI P2=LO P3=LO
        DUS #THETA_OVERLAP
        LPP #PAR_6
        LSR #CCDCLK                 * P1=HI P2=LO P3=LO SER=static
        DUS #SER_OVERLAP
*        LPP #PAR_7
*        LSR #CCDCLK                 * P1=LO P2=HI P3=LO
*        DUS #THETA_OVERLAP
        LPE
        RET

* END OF PROGRAM SEGMENT

